/*** @file   ibuy_io.c 
     @brief  iBuy File input and output functions implementation
     @author Juan Gomez Romero
     @date   2012-2013
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "structures.h"
#include "ibuy_io.h"

/******************* DO NOT MODIFY ********************/

int readUsersFromFile(char inputFile[], struct User users[], int *nUsers) {
    
    FILE *ifp;  // file pointer
    char *mode = "r", line[BUFSIZ], item[BUFSIZ];  //  temporal strings
    int indexLine = 0, indexItem; // line and field indexes 
    const int nbItems = 5;        // number of fields per user
    size_t len;
    
    ifp = fopen(inputFile, mode);
    if (ifp == NULL) {
        fprintf(stderr, "Cannot open input file: %s\n", inputFile);
        return 1;
    }
    
    while ( fgets(line, sizeof line, ifp) != NULL ) {
        // For each line...
        char *token = line; // line is processed as a pointer to char
        indexItem = 0;      // index counter is initialized to 0
        
        while ( 1 ) {
            len = strcspn(token, ";\n"); // field delimiters (; and \n) are defined
            sprintf(item, "%.*s", (int)len, token); // field data is copied into a char[]    
            token += len; // the pointer is moved forward 
            if ( *token == '\0' )
                break; // end of string 
            ++token;   // skip ';' delimiter 
            
            switch (indexItem) { 
              // read data is stored into the structure according to indexItem
            
              case 0: // login
                   strcpy(users[indexLine].login, item);
                   break;
              case 1: // password
                   strcpy(users[indexLine].password, item);
                   break;
              case 2: // name
                   strcpy(users[indexLine].name, item);
                   break;
              case 3: // surname
                   strcpy(users[indexLine].surname, item);
                   break;
              case 4: // favorites' list
                   parseIntSeq(item, users[indexLine].favoriteSellers, MAX_USERS, &users[indexLine].nFavoriteSellers);
            }

            if ( (++indexItem == nbItems) ) {
                // all fields have been read
                users[indexLine].userID = indexLine+1;   // set user id
                break; 
            }
          }

          if ( (++indexLine == MAX_USERS) )
              break; // MAX_USERS is reached, stop
    }
    
    fclose(ifp);
    
    *nUsers = indexLine;
    return 0;
}

int writeUsersToFile(char outputFile[], struct User users[], int nUsers) {
    FILE *ofp;
    char *mode = "w";
    int i, j;
   
    ofp = fopen(outputFile, mode); 
    if (ofp == NULL) {
        fprintf(stderr, "Cannot open output file: %s\n", outputFile);
        return 1;
    }
   
    for (i=0; i<nUsers; i++) {
        fprintf (ofp, "%s;%s;%s;%s;", users[i].login, users[i].password, users[i].name, users[i].surname);
        for(j=0; j<users[i].nFavoriteSellers; j++) {
            if (j !=0 ) 
                fprintf (ofp, ",");   // print ',' before the favorite user IDs but the first one
        
            fprintf (ofp, "%i", users[i].favoriteSellers[j]);
        }   
        fprintf (ofp,";\n");
    }
      
    fclose(ofp);
   
    return 0;
}

int readItemsFromFile(char inputFile[], struct Item items[], int *nItems) {
   
    FILE *ifp;
    char *mode = "r", line[BUFSIZ], item[BUFSIZ];
    int indexLine = 0, indexItem; // line and field indexes 
    const int nbItems = 7; // number of fields per item
    size_t len;
    char *stopstring;
    
    ifp = fopen(inputFile, mode);
    if (ifp == NULL) {
        fprintf(stderr, "Cannot open input file: %s\n",inputFile);
        return 1;
    }
    
    while ( fgets(line, sizeof line, ifp) != NULL ) { 
        // For each line...
        char *token = line; // line is processed as a pointer to char
        indexItem = 0;      // index counter is initialized to 0
        
        while ( 1 ) {
            len = strcspn(token, ";\n"); // field delimiters (; and \n) are defined
            sprintf(item, "%.*s", (int)len, token); // field data is copied into a char[]     
            token += len; // the pointer is moved forward 
            if ( *token == '\0' ) 
                break; // end of string  
            ++token; // skip ';' delimiter  

            switch (indexItem) {
              // read data is stored into the structure according to indexItem
              
              case 0: // description
                   strcpy(items[indexLine].description, item);
                   break;
              case 1: // name
                   strcpy(items[indexLine].name, item);                   
                   break;
              case 2: // price
                   items[indexLine].price = strtof(item, &stopstring);
                   break;              
              case 3: // seller
                   items[indexLine].sellerID = atoi(item);
                   break;              
              case 4: // like votes
                   parseIntSeq(item, items[indexLine].likeRatings, MAX_USERS, &items[indexLine].nLikeRatings);
                   break;
              case 5: // deal votes
                   parseIntSeq(item, items[indexLine].dealRatings, MAX_USERS, &items[indexLine].nDealRatings);
                   break;
              case 6: // inappropriate votes
                   parseIntSeq(item, items[indexLine].soldRatings, MAX_USERS, &items[indexLine].nSoldRatings);
            }

            if ( (++indexItem == nbItems) ) {
                items[indexLine].itemID = indexLine+1;   // set item id            
                break; // all fields have been read
            }

          }

          if ( (++indexLine == MAX_ITEMS) )
              break; // MAX_ITEMS is reached, stop
    }
    
    fclose(ifp);
    *nItems = indexLine;
   
    return 0;    
}


int writeItemsToFile(char outputFile[], struct Item items[], int nItems) {
    FILE *ofp;
    char *mode = "w";
    int i,j;
    ofp = fopen(outputFile, mode);
    
    if (ofp == NULL) {
        fprintf(stderr, "Cannot open output file: %s\n", outputFile);
        return 1;
    }
   
   for(i=0; i<nItems; i++){
       fprintf (ofp, "%s;%s;%.2f;%i;", items[i].description, items[i].name, items[i].price, items[i].sellerID);      
       for(j=0; j<items[i].nLikeRatings; j++) {        
           if (j!=0)
               fprintf (ofp,","); 
           fprintf(ofp, "%i", items[i].likeRatings[j]);
       }   
      
       fprintf (ofp,";");
       for(j=0; j<items[i].nDealRatings; j++) {
           if (j!=0)
               fprintf (ofp,",");
           fprintf (ofp, "%i", items[i].dealRatings[j]);
       }   
      
       fprintf (ofp,";");
       for(j=0; j<items[i].nSoldRatings; j++) {
           if (j!=0)
               fprintf (ofp,",");
           fprintf (ofp, "%i", items[i].soldRatings[j]);
       }   
       fprintf (ofp,";\n");      
   }
      
   fclose(ofp);
   return 0;
}

/// Internal function to parse a sequence of integer values  separated by the ',' character
int parseIntSeq(char *token, int vector[], int lVector, int *n){
    size_t len;
    int index = 0;
    char item[3];
    *n=0;
    while ( 1 ) {
        len = strcspn(token, ","); // field delimiter (,) is defined
        
        if(len > 0) {
            sprintf(item, "%.*s", (int)len, token); // process item
            vector[index] = atoi(item); // the char is transformed into an integer and stored
            token += len; // move forward token pointer
            if ( *token == '\0' ) {
               ++index;
               break; // no more items
            }
            ++token; // skip delimiter

            if ( (++index == lVector) )
               break; // maximum number of expected elements is reached
        } else {
            break; // last element was read
        }
    }
    
    *n = index;
    return 0;
};

/******************* DO NOT MODIFY ********************/
/******************* DO NOT MODIFY ********************/
