/** @file   ibuy_io.h
    @brief  iBuy file read/write functions prototypes
    @author Juan Gomez Romero
    @date   2012-2013
*/

#include "structures.h"

/******************* DO NOT MODIFY ********************/

/** @fn int readUsersFromFile(char inputFile[], struct User users[], int *nUsers);
    @brief Read data from a file to initialize User structures.            
    @param[in]  inputFile File name string
    @param[out] users     Array of User structures
    @param[out] nUsers    Number of read users
    
    @return 0 if success, 1 if error        
    
    @details The function reads a text file line-by-line. The administrator 
    user data IS NOT expected to be in the file. The function reads MAX_USERS 
    elements at most. Lines have the format:
    Login;Password;Name;Surname;Favorite_User_1_UUID,Favorite_User_2_UUID,...;
    
    Example:
    Lolmar;qwertyui;Lolencio;Martinez;1,4;
*/
int readUsersFromFile(char inputFile[], struct User users[], int *nUsers);

/*! Write data from User structures to a file    
    @param[in]  outputFile  File name string
    @param[in]  users       Array of User structures
    @param[in]  nUsers      Number of elements in array users
    
    @return 0 if success, 1 if error
    
    @detailes The function writes users into a text file line-by-line.
    The administrator user IS NOT printed into the file. Lines have the 
    format:
    Login;Password;Name;Surname;Favorite_User_1_UUID,Favorite_User_2_UUID,...;
    
    Example:
    Lolmar;qwertyui;Lolencio;Martinez;1,4;   
*/
int writeUsersToFile(char outputFile[], struct User users[], int nUsers);

/*! Read data from a file to initialize Item structures    
    @param[in]  inputFile    File name string
    @param[out] items        Array of Item structures
    @param[out] nItems       Number of read items
    
    @return 0 if success, 1 if error
    
    @details  The function reads a text file line-by-line. The function reads 
    MAX_ITEMS elements at most. Lines have the format:
    Description;Name;Price;ID_seller;ID_user_voted_Like_1, ..., ID_user_voted_Like_N;ID_user_voted_Deal_1,...,ID_user_voted_Deal_M;ID_user_voted_Sold_1,...,ID_user_voted_Sold_P;
    
    Example:
    A new brand real flux capacitor;Item1;182.64;2;1,2;3,4;5,6;
*/
int readItemsFromFile(char inputFile[], struct Item items[], int *nItems);

/*! Write data from Item structures to a file    
    @param[in] outputFile  File name string
    @param[in] items       Array of Item structures
    @param[in] nItems      Number of elements in array items
    
    @return 0 if success, 1 if error
    
    @details The function writes item data into a text file line-by-line.
    Lines have the format:
    Description;Name;Price;ID_seller;ID_user_voted_Like_1, ..., ID_user_voted_Like_N;ID_user_voted_Deal_1,...,ID_user_voted_Deal_M;ID_user_voted_Sold_1,...,ID_user_voted_Sold_P;
    
    Example:
    A new brand real flux capacitor;Item1;182.64;2;1,2;3,4;5,6;
**/
int writeItemsToFile(char outputFile[], struct Item items[], int nItems);

/******************* DO NOT MODIFY ********************/
